/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.client.api;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.ServiceConfigurationError;
import java.util.ServiceLoader;

import org.ow2.petals.registry.client.api.exception.ConnectionErrorException;
import org.ow2.petals.registry.client.api.exception.DuplicatedServiceException;
import org.ow2.petals.registry.client.api.exception.MissingServiceException;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Factory to create a client of the Petals registry cluster
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class PetalsRegistryClientFactory {

    /**
     * Name of the property used to select the implementation of Petals Registry Client API.
     */
    public static final String PROPERTY_NAME = PetalsRegistryClientFactory.class.getName();

    /**
     * <p>
     * Create a new instance of Petals Registry Client API using an implementation defined according to the following
     * rule:
     * <ul>
     * <li>first, if the property {@value #PROPERTY_NAME} is set, its value is used to look for the service
     * implementation,</li>
     * <li>otherwise the service implementation is looking for in the current classloader.</li>
     * </ul>
     * </p>
     * 
     * @throws DuplicatedServiceException
     *             The service to look for is duplicated.
     * @throws MissingServiceException
     *             Unable to find the service.
     */
    public static final PetalsRegistryClientFactory newInstance() throws DuplicatedServiceException,
            MissingServiceException {
        final PetalsRegistryClientFactory petalsRegistryClientFactory;
        final String overridenImplementation = System.getProperty(PetalsRegistryClientFactory.PROPERTY_NAME);
        if (overridenImplementation == null || overridenImplementation.trim().isEmpty()) {
            final ServiceLoader<PetalsRegistryClientFactory> serviceLoader = ServiceLoader
                    .load(PetalsRegistryClientFactory.class);
            final Iterator<PetalsRegistryClientFactory> servicesIterator = serviceLoader.iterator();
            try {
                if (servicesIterator.hasNext()) {
                    final PetalsRegistryClientFactory serviceLoaded = servicesIterator.next();
                    if (servicesIterator.hasNext()) {
                        throw new DuplicatedServiceException(PetalsRegistryClientFactory.class);
                    } else {
                        petalsRegistryClientFactory = serviceLoaded;
                    }
                } else {
                    throw new MissingServiceException(PetalsRegistryClientFactory.class);
                }
            } catch (final ServiceConfigurationError e) {
                throw new MissingServiceException(PetalsRegistryClientFactory.class, e);
            }
        } else {
            try {
                petalsRegistryClientFactory = ReflectionHelper.newInstance(overridenImplementation, null);
            } catch (final InvocationTargetException e) {
                throw new MissingServiceException(PetalsRegistryClientFactory.class, e);
            }
        }

        return petalsRegistryClientFactory;
    }

    /**
     * Create a new Petals Registry client connected to the remote Petals Registry cluster.
     * 
     * @param host
     *            A hostname where the remote Petals Registry cluster runs. If set to <code>null</code>, the host name
     *            is <code>localhost</code>.
     * @param port
     *            The port of the remote Petals Registry cluster. If set to <code>null</code>, the default port
     *            {@link PetalsRegistryClient#DEFAULT_PORT} is used.
     * @param group
     *            Group name of the credentials required to establish a connection to the remote Petals Registry
     *            cluster. If set to <code>null</code>, the default value is used:
     *            {@value Constants#DEFAULT_CREDENTIALS_GROUP}.
     * @param password
     *            Password of the credentials required to establish a connection to the remote Petals Registry cluster.
     *            If set to <code>null</code>, the default value is used:
     *            {@value Constants#DEFAULT_CREDENTIALS_PASSWORD}.
     * 
     * @return a Petals Registry client connected to the remote Petals Registry cluster
     * @throws AuthenticationException
     *             When credentials are rejected by the Petals Registry cluster
     * @throws ConnectionErrorException
     *             An error occurs establishing the connection to the Petals Registry cluster
     */
    public abstract PetalsRegistryClient createClient(final String host, final Integer port, final String group,
            final String password) throws ConnectionErrorException;
}
