/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.client.api;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;

import org.junit.Test;
import org.ow2.petals.registry.client.api.exception.DuplicatedServiceException;
import org.ow2.petals.registry.client.api.exception.MissingServiceException;

/**
 * Unit tests of {@link PetalsRegistryClientFactory}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class PetalsRegistryClientFactoryTest {

    /**
     * Dummy implementation of {@link etalsRegistryClientFactory} for duplication test.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public class PetalsRegistryClientFactoryDummy extends PetalsRegistryClientFactory {

        @Override
        public PetalsRegistryClient createClient(final String host, final Integer port, final String group,
                final String password) {
            return null;
        }
    }

    /**
     * <p>
     * Check the loading of the Petals Registry Client API implementation when only one implementation is available on
     * the classpath.
     * </p>
     * <p>
     * Expected result: The implementation available on the classpath is loaded.
     * </p>
     */
    @Test
    public void onlyOneImplAvailableOnClasspath() throws DuplicatedServiceException, MissingServiceException {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + PetalsRegistryClientFactory.class.getName())) {
                    return super.getResources("services/mock-impl/" + PetalsRegistryClientFactory.class.getName());
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            final PetalsRegistryClientFactory petalsRegistryClientFactory = PetalsRegistryClientFactory.newInstance();
            assertTrue(petalsRegistryClientFactory instanceof DummyPetalsRegistryClientFactory);
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }

    /**
     * <p>
     * Check the loading of the Petals Registry Client API implementation when no implementation is available on the
     * classpath and the property is not set.
     * </p>
     * <p>
     * Expected result: The exception {@link MissingServiceException} is thrown.
     * </p>
     */
    @Test(expected = MissingServiceException.class)
    public void noImplAvailable() throws DuplicatedServiceException, MissingServiceException {

        PetalsRegistryClientFactory.newInstance();
        fail("Exception " + MissingServiceException.class.getName() + " is not thrown !");
    }

    /**
     * <p>
     * Check the loading of the Petals Registry Client API implementation when no implementation is available on the
     * classpath and the property is set.
     * </p>
     * <p>
     * Expected result: The implementation defined by the property is loaded.
     * </p>
     */
    @Test
    public void implSetUsingProperty() throws DuplicatedServiceException, MissingServiceException {

        System.setProperty(PetalsRegistryClientFactory.PROPERTY_NAME, DummyPetalsRegistryClientFactory.class.getName());
        try {
            final PetalsRegistryClientFactory petalsRegistryClientFactory = PetalsRegistryClientFactory.newInstance();
            assertTrue(petalsRegistryClientFactory instanceof DummyPetalsRegistryClientFactory);
        } finally {
            System.setProperty(PetalsRegistryClientFactory.PROPERTY_NAME, "");
        }
    }

    /**
     * <p>
     * Check the loading of the Petals Registry Client API implementation when several implementations are available on
     * the classpath.
     * </p>
     * <p>
     * Expected result: The exception {@link DuplicatedServiceException} is thrown.
     * </p>
     */
    @Test(expected = DuplicatedServiceException.class)
    public void implDuplicatedOnClassPath() throws DuplicatedServiceException, MissingServiceException {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + PetalsRegistryClientFactory.class.getName())) {
                    return super.getResources("services/duplicated/" + PetalsRegistryClientFactory.class.getName());
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            PetalsRegistryClientFactory.newInstance();
            fail("Exception " + DuplicatedServiceException.class.getName() + " is not thrown !");
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }

    /**
     * <p>
     * Check the loading of the Petals Registry Client API implementation when only one implementation is defined into
     * the classpath, but the class does not exist.
     * </p>
     * <p>
     * Expected result: The exception {@link MissingServiceException} is thrown.
     * </p>
     */
    @Test(expected = MissingServiceException.class)
    public void implDefinedOnClassPathButClassIsMissing() throws DuplicatedServiceException, MissingServiceException {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + PetalsRegistryClientFactory.class.getName())) {
                    return super.getResources("services/class-doesnot-exist/"
                            + PetalsRegistryClientFactory.class.getName());
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            PetalsRegistryClientFactory.newInstance();
            fail("Exception " + MissingServiceException.class.getName() + " is not thrown !");
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }
}
